package com.loganmagnan.lasers;

import com.loganmagnan.lasers.managers.CooldownManager;
import com.loganmagnan.lasers.menusystem.PlayerMenuUtil;
import com.loganmagnan.lasers.runnables.CooldownRunnable;
import com.loganmagnan.lasers.utils.ClassRegistrationUtils;
import com.loganmagnan.lasers.utils.ColorUtils;
import com.loganmagnan.lasers.utils.ItemBuilder;
import com.loganmagnan.lasers.utils.Utils;
import com.loganmagnan.lasers.utils.command.CommandFramework;
import com.loganmagnan.lasers.utils.config.FileConfig;
import com.loganmagnan.lasers.utils.config.file.Config;
import lombok.Getter;
import lombok.Setter;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.ShapedRecipe;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.plugin.java.JavaPlugin;

import java.util.*;

@Getter
@Setter
public class Lasers extends JavaPlugin {

    // Main Class Instance
    @Getter private static Lasers instance;

    // Configuration Files
    private Config mainConfig;
    private FileConfig messagesConfig;
    private FileConfig envoysConfig;

    // Managers
    private CooldownManager cooldownManager;

    // Menu System
    private HashMap<Player, PlayerMenuUtil> playerMenuUtilMap = new HashMap<>();

    // Command Framework
    private CommandFramework commandFramework = new CommandFramework(this);

    @Override
    public void onEnable() {
        instance = this;

        this.saveDefaultConfig();
        this.mainConfig = new Config("config", this);
        this.messagesConfig = new FileConfig(this, "messages.yml");

        this.getServer().getConsoleSender().sendMessage(Utils.chatBar);
        this.getServer().getConsoleSender().sendMessage(ColorUtils.getMessageType("&dLasers &7- &av" + this.getDescription().getVersion()));
        this.getServer().getConsoleSender().sendMessage(ColorUtils.getMessageType("&7Made by &eLoganM Development"));
        this.getServer().getConsoleSender().sendMessage(Utils.chatBar);
        this.loadCommands();
        this.loadManagers();
        this.loadListeners();
        this.loadRunnables();

        ItemStack itemStack = new ItemBuilder(Material.DIAMOND_HORSE_ARMOR).name(ColorUtils.getMessageType("&4&lLaser")).lore(Arrays.asList(ColorUtils.getMessageType("&7&oRight click to use"))).build();

        ShapedRecipe shapedRecipe = new ShapedRecipe(itemStack);
        shapedRecipe.shape("ABA", "BCB", "ABA");
        shapedRecipe.setIngredient('A', Material.GOLD_INGOT);
        shapedRecipe.setIngredient('B', Material.IRON_INGOT);
        shapedRecipe.setIngredient('C', Material.DIAMOND);

        this.getServer().addRecipe(shapedRecipe);
    }

    @Override
    public void onDisable() {
        instance = null;
    }

    private void loadCommands() {
        ClassRegistrationUtils.loadCommands("com.loganmagnan.lasers.commands");
    }

    private void loadManagers() {
        this.cooldownManager = new CooldownManager();
    }

    private void loadListeners() {
        ClassRegistrationUtils.loadListeners("com.loganmagnan.lasers.listeners");
    }

    private void loadRunnables() {
        this.getServer().getScheduler().runTaskTimerAsynchronously(this, new CooldownRunnable(), 0, 20);
    }

    public PlayerMenuUtil getPlayerMenuUtil(Player player) {
        PlayerMenuUtil playerMenuUtil;

        if (playerMenuUtilMap.containsKey(player)) {
            return playerMenuUtilMap.get(player);
        } else {
            playerMenuUtil = new PlayerMenuUtil(player);

            playerMenuUtilMap.put(player, playerMenuUtil);

            return playerMenuUtil;
        }
    }
}
