package com.loganmagnan.lasers.listeners;

import com.loganmagnan.lasers.Lasers;
import com.sun.tools.javac.util.Names;
import org.bukkit.*;
import org.bukkit.entity.Entity;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.util.Vector;

import java.util.Iterator;

public class PlayerInteractListener implements Listener {

    private Lasers main = Lasers.getInstance();

    @EventHandler
    public void onPlayerInteract(PlayerInteractEvent event) {
        Player player = event.getPlayer();

        if (!player.getItemInHand().hasItemMeta() && player.getItemInHand().getType() != Material.DIAMOND_HORSE_ARMOR) {
            return;
        }

        if (event.getAction() != Action.RIGHT_CLICK_AIR && event.getAction() != Action.RIGHT_CLICK_BLOCK) {
            return;
        }

        if (this.main.getCooldownManager().isOnCooldown(player.getUniqueId())) {
            return;
        }

        Location location = player.getLocation().clone().add(0, 1, 0);

        this.main.getCooldownManager().setCooldown(player.getUniqueId(), 11);
        this.spawnParticle(player, location);
    }

    public Iterable<Vector> getVectorList(Vector startVector, Vector endVector) {
        return () -> new Iterator<Vector>() {
            private Vector currentVector = startVector.clone();
            private Vector direction = endVector.clone().subtract(startVector).normalize();

            private double length = startVector.distance(endVector);
            private double stepSize = 0.10;

            @Override
            public boolean hasNext() {
                return currentVector.distance(endVector) <= length;
            }

            @Override
            public Vector next() {
                Vector vector = currentVector.clone();
                currentVector.add(direction.clone().multiply(stepSize));

                return vector;
            }
        };
    }

    public void spawnParticle(Player player, Location playerLocation) {
        Location endLocation = playerLocation.toVector().add(playerLocation.getDirection().normalize().multiply(20)).toLocation(playerLocation.getWorld());

        Particle.DustOptions dustOptions = new Particle.DustOptions(Color.fromRGB(255, 0, 0), 5);

        for (Vector vectorInList : this.getVectorList(playerLocation.toVector(), endLocation.toVector())) {
            Location location = vectorInList.toLocation(playerLocation.getWorld());
            location.getWorld().spawnParticle(Particle.REDSTONE, vectorInList.toLocation(playerLocation.getWorld()), 1, dustOptions);

            for (Entity entity : playerLocation.getNearbyEntities(100, 100, 100)) {
                if (!(entity instanceof LivingEntity)) {
                    continue;
                }

                LivingEntity livingEntity = (LivingEntity) entity;

                if (player.getName().equalsIgnoreCase(livingEntity.getName())) {
                    continue;
                }

                if (livingEntity.getLocation().distanceSquared(location) < 1) {
                    livingEntity.damage(4);
                }
            }
        }
    }
}
